@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@ Launcher
 import os
 import sys
 
+have_gui = True
 
 if 'posix' == os.name:
     if os.path.isdir('/usr/share/bleachbit'):
@@ -34,7 +35,7 @@ if 'posix' == os.name:
 
     # The two imports from bleachbit must come after sys.path.append(..)
     import bleachbit.Unix
-    from bleachbit import _
+    from bleachbit.Language import get_text as _
 
     if (
         bleachbit.Unix.is_display_protocol_wayland_and_root_not_allowed()
@@ -43,19 +44,32 @@ if 'posix' == os.name:
               'xhost si:localuser:root\n'
                 'See more about xhost at https://docs.bleachbit.org/doc/frequently-asked-questions.html'))
         sys.exit(1)
+    # Check for GUI only when needed: this avoids a Gtk warning when
+    # a display is not available.
+    if len(sys.argv) == 1 or '--gui' in sys.argv:
+        have_gui = bleachbit.Unix.has_gui()
+    else:
+        have_gui = False
 
 if os.name == 'nt':
     # change error handling to avoid popup with GTK 3
     # https://github.com/bleachbit/bleachbit/issues/651
-    import win32api
-    import win32con
+    import win32api  # pylint: disable=import-error
+    import win32con  # pylint: disable=import-error
     win32api.SetErrorMode(win32con.SEM_FAILCRITICALERRORS |
                           win32con.SEM_NOGPFAULTERRORBOX | win32con.SEM_NOOPENFILEERRORBOX)
 
-if 1 == len(sys.argv):
-    import bleachbit.GUI
+# Use GUI if no arguments provided and display is available
+if 1 == len(sys.argv) and have_gui:
+    # Import GUI inside the condition for Linux packagers to
+    # separate GUI into another package.
+    import bleachbit.GUI  # pylint: disable=ungrouped-imports
     app = bleachbit.GUI.Bleachbit()
     sys.exit(app.run(sys.argv))
 else:
+    # Either CLI args were provided or no display is available
     import bleachbit.CLI
+    # If no args and defaulting to CLI, print usage information
+    if 1 == len(sys.argv) and not have_gui:
+        sys.argv.append('--help')
     bleachbit.CLI.process_cmd_line()
