@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,16 +23,23 @@
 Actions that perform cleaning
 """
 
-from bleachbit import Command, FileUtilities, General, Special, DeepScan
-from bleachbit import _, fs_scan_re_flags
-
+# standard imports
 import glob
 import logging
 import os
 import re
-if 'posix' == os.name:
+from itertools import product
+
+# first party imports
+from bleachbit import Command, FileUtilities, General, Special, DeepScan
+from bleachbit import fs_scan_re_flags
+from bleachbit.Language import get_text as _
+
+if os.name == 'posix':
     from bleachbit import Unix
 
+if os.name == 'nt':
+    from bleachbit import Windows
 
 logger = logging.getLogger(__name__)
 
@@ -55,7 +62,7 @@ def expand_multi_var(s, variables):
     var_keys_used = []
     ret = []
     for var_key in variables.keys():
-        sub = '$$%s$$' % var_key
+        sub = f'$${var_key}$$'
         if s.find(sub) > -1:
             var_keys_used.append(var_key)
     if not var_keys_used:
@@ -65,13 +72,12 @@ def expand_multi_var(s, variables):
     vars_used = {key: value for key,
                  value in variables.items() if key in var_keys_used}
     # create a product of combinations
-    from itertools import product
     vars_product = (dict(zip(vars_used, x))
                     for x in product(*vars_used.values()))
     for var_set in vars_product:
         ms = s  # modified version of input string
         for var_key, var_value in var_set.items():
-            sub = '$$%s$$' % var_key
+            sub = f'$${var_key}$$'
             ms = ms.replace(sub, var_value)
         ret.append(ms)
     if ret:
@@ -89,7 +95,7 @@ class PluginMount(type):
 
     """A simple plugin framework"""
 
-    def __init__(cls, name, bases, attrs):
+    def __init__(cls, _name, _bases, _attrs):
         if not hasattr(cls, 'plugins'):
             cls.plugins = []
         else:
@@ -126,13 +132,13 @@ class FileActionProvider(ActionProvider):
         """Initialize file search"""
         ActionProvider.__init__(self, action_element, path_vars)
         self.regex = action_element.getAttribute('regex')
-        assert(isinstance(self.regex, (str, type(None))))
+        assert (isinstance(self.regex, (str, type(None))))
         self.nregex = action_element.getAttribute('nregex')
-        assert(isinstance(self.nregex, (str, type(None))))
+        assert (isinstance(self.nregex, (str, type(None))))
         self.wholeregex = action_element.getAttribute('wholeregex')
-        assert(isinstance(self.wholeregex, (str, type(None))))
+        assert (isinstance(self.wholeregex, (str, type(None))))
         self.nwholeregex = action_element.getAttribute('nwholeregex')
-        assert(isinstance(self.nwholeregex, (str, type(None))))
+        assert (isinstance(self.nwholeregex, (str, type(None))))
         self.search = action_element.getAttribute('search')
         self.object_type = action_element.getAttribute('type')
         self._set_paths(action_element.getAttribute('path'), path_vars)
@@ -144,9 +150,11 @@ class FileActionProvider(ActionProvider):
                 wholeregex=self.wholeregex, nwholeregex=self.nwholeregex))
             if len(self.paths) != 1:
                 logger.warning(
-                    # TRANSLATORS: Multi-value variables are explained in the online documentation.
-                    # Basically, they are like an environment variable, but each multi-value variable
-                    # can have multiple values. They're a way to make CleanerML files more concise.
+                    # TRANSLATORS: Multi-value variables are explained
+                    # in the online documentation. Basically, they are like
+                    # an environment variable, but each multi-value variable
+                    # can have multiple values. They're a way to make CleanerML
+                    # files more concise.
                     _("Deep scan does not support multi-value variable."))
         if not any([self.object_type, self.regex, self.nregex,
                     self.wholeregex, self.nwholeregex]):
@@ -195,7 +203,8 @@ class FileActionProvider(ActionProvider):
             nregex_c_search = None
 
         if self.wholeregex:
-            wholeregex_c_search = re.compile(self.wholeregex, fs_scan_re_flags).search
+            wholeregex_c_search = re.compile(
+                self.wholeregex, fs_scan_re_flags).search
         else:
             wholeregex_c_search = None
 
@@ -276,7 +285,7 @@ class FileActionProvider(ActionProvider):
         }
 
         if self.search not in search_functions:
-            raise RuntimeError("Invalid search='%s'" % self.search)
+            raise RuntimeError(f"Invalid search='{self.search}'")
 
         func = search_functions[self.search]
 
@@ -292,7 +301,7 @@ class FileActionProvider(ActionProvider):
 
             # use cache
             if self.search in self.CACHEABLE_SEARCHERS and cache[0] == self.search and cache[1] == input_path:
-                #logger.debug(_('using cached walk for path %s'), input_path)
+                # logger.debug(_('using cached walk for path %s'), input_path)
                 for x in cache[2]:
                     yield x
                 return
@@ -302,7 +311,7 @@ class FileActionProvider(ActionProvider):
             self.__class__.cache = ('cleared by', input_path, tuple())
 
             # build new cache
-            #logger.debug('%s walking %s', id(self), input_path)
+            # logger.debug('%s walking %s', id(self), input_path)
 
             if self.search in self.CACHEABLE_SEARCHERS:
                 cache = self.__class__.cache = (self.search, input_path, [])
@@ -332,7 +341,9 @@ class AptAutoclean(ActionProvider):
 
     def get_commands(self):
         # Checking executable allows auto-hide to work for non-APT systems
+        assert os.name == 'posix'
         if FileUtilities.exe_exists('apt-get'):
+            # pylint: disable=possibly-used-before-assignment
             yield Command.Function(None,
                                    Unix.apt_autoclean,
                                    'apt-get autoclean')
@@ -544,21 +555,17 @@ class Process(ActionProvider):
 
         def run_process():
             try:
-                if self.wait:
-                    args = self.cmd.split(' ')
-                    (rc, stdout, stderr) = General.run_external(args)
-                else:
-                    rc = 0  # unknown because we don't wait
-                    from subprocess import Popen
-                    Popen(self.cmd)
+                args = General.shell_split(self.cmd)
+                (rc, stdout, stderr) = General.run_external(args, wait=self.wait)
             except Exception as e:
                 raise RuntimeError(
-                    'Exception in external command\nCommand: %s\nError: %s' % (self.cmd, str(e))) from e
-            if 0 != rc:
-                msg = 'Command: %s\nReturn code: %d\nStdout: %s\nStderr: %s\n'
-                logger.warning(msg, self.cmd, rc, stdout, stderr)
+                    f'Exception in external command\nCommand: {args}\nError: {str(e)}') from e
+            if self.wait and 0 != rc:
+                logger.warning('Command: %s\nReturn code: %d\nStdout: %s\nStderr: %s\n',
+                               args, rc, stdout, stderr)
             return 0
-        yield Command.Function(path=None, func=run_process, label=_("Run external command: %s") % self.cmd)
+        yield Command.Function(path=None, func=run_process,
+                               label=_("Run external command: %s") % self.cmd)
 
 
 class Shred(FileActionProvider):
@@ -604,11 +611,12 @@ class WinShellChangeNotify(ActionProvider):
     action_key = 'win.shell.change.notify'
 
     def get_commands(self):
-        from bleachbit import Windows
+        assert os.name == 'nt'
         yield Command.Function(
             None,
+            # pylint: disable=possibly-used-before-assignment
             Windows.shell_change_notify,
-            None)
+            _('Refresh Windows shell'))
 
 
 class Winreg(ActionProvider):
@@ -680,3 +688,34 @@ class DnfAutoremove(ActionProvider):
             None,
             Unix.dnf_autoremove,
             'dnf autoremove')
+
+
+class PacmanCache(ActionProvider):
+
+    """Action to run `paccache -rk0'"""
+    action_key = 'pacman.cache'
+
+    def __init__(self, action_element, path_vars=None):
+        ActionProvider.__init__(self, action_element, path_vars)
+
+    def get_commands(self):
+        yield Command.Function(
+            None,
+            Unix.pacman_cache,
+            'paccache -rk0')
+
+
+class SnapDisabled(ActionProvider):
+
+    """Action to remove disabled snaps"""
+    action_key = 'snap.disabled'
+
+    def __init__(self, action_element, path_vars=None):
+        ActionProvider.__init__(self, action_element, path_vars)
+
+    def get_commands(self):
+        yield Command.Function(
+            None,
+            Unix.snap_disabled_clean,
+            'snap remove disabled',
+            Unix.snap_disabled_preview)
