@@ -1,8 +1,7 @@
-#!/usr/bin/python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,8 +22,9 @@ Command line interface
 """
 
 from bleachbit.Cleaner import backends, create_simple_cleaner, register_cleaners
-from bleachbit import _, APP_VERSION
+from bleachbit import APP_VERSION
 from bleachbit import SystemInformation, Options, Worker
+from bleachbit.Language import get_text as _
 from bleachbit.Log import set_root_log_level
 
 import logging
@@ -78,8 +78,13 @@ def preview_or_clean(operations, really_clean, quiet=False):
     """Preview deletes and other changes"""
     cb = CliCallback(quiet)
     worker = Worker.Worker(cb, really_clean, operations).run()
-    while next(worker):
+    try:
+        while next(worker):
+            pass
+    except StopIteration:
         pass
+    except Exception:
+        logger.exception('Failed to clean')
 
 
 def args_to_operations_list(preset, all_but_warning):
@@ -206,11 +211,6 @@ def process_cmd_line():
             _('Specify only one of these commands: --list-cleaners, --wipe-free-space, --preview, --clean'))
         sys.exit(1)
 
-    if not options.gui:
-        # The GUI has its own trigger for the same function.
-        from bleachbit.General import startup_check
-        startup_check()
-
     did_something = False
     if options.debug:
         # set in __init__ so it takes effect earlier
@@ -221,12 +221,13 @@ def process_cmd_line():
             set_root_log_level(Options.options.get('debug'))
             logger.debug("Debugging is enabled in GUI settings.")
     if options.debug_log:
-        logger.addHandler(logging.FileHandler(options.debug_log))
+        # File handler is already set up in Log.py init_log() function
+        # Just add system information to the log
         logger.info(SystemInformation.get_system_information())
     if options.version:
         print("""
 BleachBit version %s
-Copyright (C) 2008-2024 Andrew Ziem.  All rights reserved.
+Copyright (C) 2008-2025 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
@@ -263,16 +264,13 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         if not operations:
             logger.error(_("No work to do. Specify options."))
             sys.exit(1)
-    if options.preview:
-        preview_or_clean(operations, False)
-        sys.exit(0)
     if options.overwrite:
         if not options.clean or options.shred:
             logger.warning(
                 _("--overwrite is intended only for use with --clean"))
         Options.options.set('shred', True, commit=False)
-    if options.clean:
-        preview_or_clean(operations, True)
+    if options.clean or options.preview:
+        preview_or_clean(operations, options.clean)
         sys.exit(0)
     if options.gui:
         import bleachbit.GUI
