@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,49 +23,19 @@
 Test case for module Chaff
 """
 
-import mock
+from unittest import mock
 import os
 from tempfile import mkdtemp
 from shutil import rmtree
 
 from tests import common
-from bleachbit.Chaff import download_models, generate_emails, generate_2600, have_models
+from bleachbit.Chaff import download_models, generate_emails, generate_2600, have_models, MODEL_BASENAMES, DEFAULT_MODELS_DIR
 from bleachbit.FileUtilities import getsize
 
 
 class ChaffTestCase(common.BleachbitTestCase):
     """Test case for module Chaff"""
 
-    def test_download_url_to_fn(self):
-        """Unit test for function download_url_to_fn()"""
-        from bleachbit.Chaff import download_url_to_fn
-        # 200: no error
-        # 404: not retryable error
-        # 503: retryable error
-        tests = (('http://httpstat.us/200', True),
-                 ('http://httpstat.us/404', False),
-                 ('http://httpstat.us/503', False))
-        fn = os.path.join(self.tempdir, 'download')
-        on_error_called = [False]
-
-        def on_error(msg1, msg2):
-            print('test on_error(%s, %s)' % (msg1, msg2))
-            on_error_called[0] = True
-        for test in tests:
-            self.assertNotExists(fn)
-            url = test[0]
-            expected_rc = test[1]
-            on_error_called = [False]
-            rc = download_url_to_fn(
-                url, fn, on_error=on_error, timeout=20)
-            err_msg = 'test_download_url_to_fn({}) returned {} instead of {}'.format(url, rc, expected_rc)
-            self.assertEqual(rc, expected_rc, err_msg)
-            if expected_rc:
-                self.assertExists(fn)
-            self.assertNotEqual(rc, on_error_called[0])
-            from bleachbit.FileUtilities import delete
-            delete(fn, ignore_missing=True)
-
     def test_Chaff(self):
         """Unit test for class Chaff"""
         tmp_dir = mkdtemp(prefix='bleachbit-chaff')
@@ -106,13 +76,13 @@ class ChaffTestCase(common.BleachbitTestCase):
 
         rmtree(tmp_dir)
 
-    @mock.patch('bleachbit.Chaff.download_url_to_fn')
+    @mock.patch('bleachbit.Network.download_url_to_fn')
     def test_download_models_fallback(self, mock_download):
         """Test the fallback mechanism in download_models()"""
         tmp_dir = mkdtemp(prefix='bleachbit-chaff')
 
         # Test when primary download mirror fails but secondary succeeds.
-        def succeed_on_second(*args):
+        def succeed_on_second(*args, **kwargs):
             url = args[0]
             if 'sourceforge' in url:
                 return False
@@ -134,5 +104,13 @@ class ChaffTestCase(common.BleachbitTestCase):
         rmtree(tmp_dir)
 
     def test_have_models(self):
+        """Test for function have_models()"""
+        download_models()
         rc = have_models()
+        self.assertTrue(rc)
         self.assertIsInstance(rc, bool)
+        for basename in MODEL_BASENAMES:
+            fn = os.path.join(DEFAULT_MODELS_DIR, basename)
+            self.assertExists(fn)
+            os.remove(fn)
+        self.assertFalse(have_models())
