@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -144,7 +144,7 @@ class DeepScanTestCase(common.BleachbitTestCase):
 
     @unittest.skipUnless('darwin' == sys.platform, 'Not on Darwin')
     def test_normalized_walk_darwin(self):
-        import mock
+        from unittest import mock
 
         with mock.patch('os.walk') as mock_walk:
             mock_walk.return_value = [
