@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,33 +23,36 @@
 Test case for module GUI
 """
 
-import mock
 import os
 import unittest
 import time
 import types
+from unittest import mock
 
-try:
+HAVE_GTK = True
+if os.name == 'posix':
+    from bleachbit.Unix import has_gui
+    HAVE_GTK = has_gui()
+
+if HAVE_GTK:
     import gi
     gi.require_version('Gtk', '3.0')
     from gi.repository import Gtk
     from bleachbit.GUI import Bleachbit
-    HAVE_GTK = True
-except ImportError:
-    HAVE_GTK = False
 
 import bleachbit
-from bleachbit import _
+from bleachbit.Language import get_text as _
 from bleachbit.Options import options
 
+
 from tests import common
 
 bleachbit.online_update_notification_enabled = False
 
 
-@unittest.skipUnless(HAVE_GTK, 'requires GTK+ module')
+@unittest.skipUnless(HAVE_GTK, 'requires GTK+ module and a display environment')
 class GUITestCase(common.BleachbitTestCase):
-    app = Bleachbit(auto_exit=False, uac=False)
+    app = Bleachbit(auto_exit=False, uac=False) if HAVE_GTK else None
     options_get_tree = options.get_tree
     _NEW_CLEANER_ID, _NEW_OPTION_ID = 'test_run_operations', 'test1'
 
@@ -110,7 +113,7 @@ class GUITestCase(common.BleachbitTestCase):
     @classmethod
     def find_option(cls, model, cleaner, option):
         it = cls.get_iter(model, cleaner)
-        #cls.assertIsNotNone(cls, it)
+        # cls.assertIsNotNone(cls, it)
         while it:
             if model[it][2] == option:
                 return it
@@ -134,6 +137,18 @@ class GUITestCase(common.BleachbitTestCase):
         b.clicked()
         self.refresh_gui()
 
+    def test_get_window_info(self):
+        """Test get_window_info"""
+        gui = self.app._window
+        geo = bleachbit.GUI.get_window_info(gui)
+        self.assertGreaterEqual(geo.x, 0)
+        self.assertGreaterEqual(geo.y, 0)
+        self.assertGreaterEqual(geo.width, 0)
+        self.assertGreaterEqual(geo.height, 0)
+        self.assertIsInstance(geo.monitor_model, str)
+        self.assertGreater(len(geo.monitor_model), 0)
+        self.assertNotEqual(geo.monitor_model, "(error)")
+
     def test_GUI(self):
         """Unit test for class GUI"""
         # there should be no crashes
@@ -144,6 +159,19 @@ class GUITestCase(common.BleachbitTestCase):
         gui.update_progress_bar(1.0)
         gui.update_progress_bar("status")
 
+    def test_get_font_size_from_name(self):
+        """Test get_font_size_from_name()"""
+        tests = (("Sans 12", 12),
+                 ("Sans 10.5", 10),
+                 ("Sans 0", None),
+                 ("Sans -1", None),
+                 ("Sans", None),
+                 ("", None),
+                 (None, None))
+        for font_name, expected in tests:
+            self.assertEqual(bleachbit.GUI.get_font_size_from_name(
+                font_name), expected, f"Font name '{font_name}' should return {expected}")
+
     def test_preferences(self):
         """Opens the preferences dialog and closes it"""
 
@@ -179,6 +207,26 @@ class GUITestCase(common.BleachbitTestCase):
         # destroy
         about.destroy()
 
+    def test_chaff(self):
+        """Minimal test of the chaff dialog"""
+        import bleachbit.GuiChaff
+        import bleachbit.Chaff
+        # common.py patches the download directory, so have_models() will return False.
+        bleachbit.Chaff.download_models()
+        gui = self.app._window
+        cd = bleachbit.GuiChaff.ChaffDialog(gui)
+        cd.show_all()
+        chaff_dst_dir = os.path.join(self.tempdir, 'chaff_dst')
+        os.mkdir(chaff_dst_dir)
+        cd.choose_folder_button.set_filename(chaff_dst_dir)
+        cd.when_finished_combo.set_active(1)  # do not delete after generation
+        cd.file_count.set_value(10)
+        self.refresh_gui()
+        self.click_button(cd, 'Make files')
+        cd.thread.join()
+        self.refresh_gui()
+        self.assertEqual(len(os.listdir(chaff_dst_dir)), 10)
+
     def test_preview(self):
         """Select cleaner option and clicks preview button"""
         gui = self.app._window
@@ -193,8 +241,7 @@ class GUITestCase(common.BleachbitTestCase):
         """Test a pop-up notification"""
         from bleachbit.GUI import notify
         notify('This is a test notification')
-        import time
-        time.sleep
+        time.sleep(1)  # Pause for 1 second
 
     @mock.patch('bleachbit.GuiBasic.delete_confirmation_dialog')
     def test_confirm_delete(self, mock_delete_confirmation_dialog):
