@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,11 +22,24 @@
 Test case for module Memory
 """
 
-from tests import common
-from bleachbit.Memory import *
-
+import os
+import re
 import unittest
 
+from tests import common
+from bleachbit import logger
+from bleachbit.Memory import (
+    count_swap_linux,
+    disable_swap_linux,
+    enable_swap_linux,
+    get_proc_swaps,
+    get_swap_size_linux,
+    get_swap_uuid,
+    make_self_oom_target_linux,
+    parse_swapoff,
+    physical_free, physical_free_darwin,
+    report_free)
+
 
 class MemoryTestCase(common.BleachbitTestCase):
     """Test case for module Memory"""
