@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2023 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 
 import os
-import mock
+from unittest import mock
 
 from windows.NsisUtilities import (_generate_add_remove_nsis_expressions,
                                    _walk_with_parent_directory_and_filepaths,
