@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -27,11 +27,11 @@ from bleachbit.Options import options
 from bleachbit import FileUtilities, Special
 from tests import common
 
+import contextlib
 import os
 import os.path
 import shutil
 import sqlite3
-import contextlib
 
 chrome_bookmarks = b"""
 {
@@ -274,13 +274,13 @@ class SpecialAssertions:
         """Asserts SQLite tables exists and are empty"""
         if not os.path.lexists(path):
             raise AssertionError('Path does not exist: %s' % path)
-        conn = sqlite3.connect(path)
-        cursor = conn.cursor()
-        for table in tables:
-            cursor.execute('select 1 from %s limit 1' % table)
-            row = cursor.fetchone()
-            if row:
-                raise AssertionError('Table is not empty: %s ' % table)
+        with contextlib.closing(sqlite3.connect(path)) as conn:
+            cursor = conn.cursor()
+            for table in tables:
+                cursor.execute('select 1 from %s limit 1' % table)
+                row = cursor.fetchone()
+                if row:
+                    raise AssertionError('Table is not empty: %s ' % table)
 
 
 class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
@@ -331,6 +331,8 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
 
     def tearDown(self):
         """Remove test browser files."""
+        from bleachbit.General import gc_collect
+        gc_collect()
         shutil.rmtree(self.dir_base)
 
     def sqlite_clean_helper(self, sql, fn, clean_func, check_func=None, setup_func=None):
@@ -395,7 +397,8 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
         def check_autofill(testcase, filename):
             testcase.assertTablesAreEmpty(
                 filename,
-                ['autofill', 'local_addresses', 'local_addresses_type_tokens', 'server_addresses']
+                ['autofill', 'local_addresses',
+                    'local_addresses_type_tokens', 'server_addresses']
             )
 
         self.sqlite_clean_helper(
@@ -411,17 +414,20 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
 
         def check_chrome_history(self, filename):
             conn = sqlite3.connect(filename)
-            c = conn.cursor()
-            c.execute('select id from urls')
-            ids = [row[0] for row in c]
-            # id 1 is sqlite.org which is not a bookmark and should be cleaned
-            # id 2 is www.bleachbit.org which is a bookmark and should
-            # be cleaned
-            self.assertEqual(ids, [2])
+            try:
+                c = conn.cursor()
+                c.execute('select id from urls')
+                ids = [row[0] for row in c]
+                # id 1 is sqlite.org which is not a bookmark and should be cleaned
+                # id 2 is www.bleachbit.org which is a bookmark and should
+                # be cleaned
+                self.assertEqual(ids, [2])
 
-            # these tables should always be empty after cleaning
-            self.assertTablesAreEmpty(filename, ['downloads', 'keyword_search_terms',
-                                                 'segment_usage', 'segments', 'visits'])
+                # these tables should always be empty after cleaning
+                self.assertTablesAreEmpty(filename, ['downloads', 'keyword_search_terms',
+                                                     'segment_usage', 'segments', 'visits'])
+            finally:
+                conn.close()
 
         self.sqlite_clean_helper(None, "google-chrome/Default/History",
                                  Special.delete_chrome_history, check_chrome_history)
@@ -431,10 +437,13 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
 
         def check_chrome_keywords(self, filename):
             conn = sqlite3.connect(filename)
-            c = conn.cursor()
-            c.execute('select id from keywords')
-            ids = [row[0] for row in c]
-            self.assertEqual(ids, [2, 3, 4, 5, 6])
+            try:
+                c = conn.cursor()
+                c.execute('select id from keywords')
+                ids = [row[0] for row in c]
+                self.assertEqual(ids, [2, 3, 4, 5, 6])
+            finally:
+                conn.close()
 
         self.sqlite_clean_helper(None, "google-chrome/Default/Web Data", Special.delete_chrome_keywords,
                                  check_chrome_keywords)
@@ -479,10 +488,10 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
             icon_ids = map(lambda x: x[0], icon_entries)
             cursor.close()
 
-        expected_icon_ids = [4, 5, 6, 7, 13, 14, 42]  # not removed ids from moz_icons table
+        # not removed ids from moz_icons table
+        expected_icon_ids = [4, 5, 6, 7, 13, 14, 42]
         assert sorted(expected_icon_ids) == sorted(icon_ids)
 
-
     def test_get_chrome_bookmark_ids(self):
         """Unit test for get_chrome_bookmark_ids()"""
         # does not exist
@@ -507,10 +516,66 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
         sql = """CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY,value LONGVARCHAR);
 INSERT INTO "meta" VALUES('version','20');"""
         # create test file
-        filename = self.mkstemp(prefix='bleachbit-test-sqlite')
+        filename = os.path.join(self.tempdir, 'test_get_sqlite_int.sqlite')
         FileUtilities.execute_sqlite3(filename, sql)
         self.assertExists(filename)
         # run the test
         ver = Special.get_sqlite_int(
             filename, 'select value from meta where key="version"')
         self.assertEqual(ver, [20])
+        os.unlink(filename)
+
+    def test_get_sqlite_values(self):
+        """Unit test for get_sqlite_values()"""
+        ddl = """CREATE TABLE foo(id int, value int);INSERT INTO foo VALUES(12, 34), (56, 78);"""
+        # create test file
+        filename = os.path.join(self.tempdir, 'test_get_sqlite_values.sqlite')
+        FileUtilities.execute_sqlite3(filename, ddl)
+        self.assertExists(filename)
+        # run the test
+        sql = 'select id, value from foo'
+        # pylint: disable=protected-access
+        ver = Special._get_sqlite_values(filename, sql)
+        self.assertEqual(ver, [(12, 34), (56, 78)])
+        ver = Special._get_sqlite_values(
+            filename, f'{sql} where id = 0')
+        self.assertEqual(ver, [])
+        with self.assertRaises(sqlite3.OperationalError):
+            Special._get_sqlite_values(
+                filename, 'select id, value from does_not_exist')
+        # If file is missing, raise exception and do not create the file.
+        non_existing_file = os.path.join(self.tempdir, 'non_existing_file')
+        with self.assertRaises(sqlite3.OperationalError):
+            Special._get_sqlite_values(non_existing_file, sql)
+        self.assertNotExists(non_existing_file)
+        os.unlink(filename)
+
+    def test_sqlite_table_exists(self):
+        """Unit test for _sqlite_table_exists()"""
+        # create test file
+        filename = os.path.join(
+            self.tempdir, 'test_sqlite_table_exists.sqlite')
+        sql = "CREATE TABLE foo(id int)"
+        FileUtilities.execute_sqlite3(filename, sql)
+        self.assertExists(filename)
+        # run the test
+        # pylint: disable=protected-access
+        self.assertTrue(Special._sqlite_table_exists(filename, 'foo'))
+        self.assertFalse(Special._sqlite_table_exists(
+            filename, 'table_does_not_exist'))
+        non_existing_file = os.path.join(self.tempdir, 'file_does_not_exist')
+        self.assertFalse(Special._sqlite_table_exists(
+            non_existing_file, 'table_does_not_exist'))
+        self.assertNotExists(non_existing_file)
+        os.unlink(filename)
+
+    def test_sqlite_loop(self):
+        """Repeat SQLite tests
+
+        This may raise a ResourceWarning on Python 3.13
+        """
+        for _ in range(100):
+            self.test_delete_chrome_autofill()
+            self.test_get_sqlite_int()
+            self.test_get_sqlite_values()
+            self.test_sqlite_table_exists()
