@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -26,7 +26,7 @@ Test case for module Update
 from tests import common
 import bleachbit
 from bleachbit import logger
-from bleachbit.Update import check_updates, get_gtk_version, get_ip_for_url, update_winapp2, user_agent
+from bleachbit.Update import check_updates, update_winapp2
 import bleachbit.Update
 
 import os
@@ -36,8 +36,10 @@ import os.path
 class UpdateTestCase(common.BleachbitTestCase):
     """Test case for module Update"""
 
-    def test_UpdateCheck_fake(self):
-        """Unit tests for class UpdateCheck using fake network"""
+    def test_check_updates_mock(self):
+        """Unit test for function check_updates() using mock"""
+        from unittest.mock import Mock, patch
+        from bleachbit.Update import check_updates
 
         wa = '<winapp2 url="http://katana.oooninja.com/bleachbit/winapp2.ini" sha512="ce9e18252f608c8aff28811e372124d29a86404f328d3cd51f1f220578744bb8b15f55549eabfe8f1a80657fc940f6d6deece28e0532b3b0901a4c74110f7ba7"/>'
         update_tests = [
@@ -49,41 +51,33 @@ class UpdateTestCase(common.BleachbitTestCase):
              (('0.8.5beta', 'http://085beta'), )),
             ('<updates></updates>', ())]
 
-        # fake network
-        original_open = bleachbit.Update.build_opener
-        xml = ""
+        with patch('bleachbit.Update.fetch_url') as mock_fetch:
+            # Configure mock responses
+            mock_response = Mock()
+            mock_response.status_code = 200
+            mock_response.content = b''
 
-        class FakeOpener:
-            def add_headers(self):
-                pass
+            for xml, expected in update_tests:
+                # Set up mock response content
+                mock_response.text = xml
+                mock_response.content = xml.encode()
+                mock_fetch.return_value = mock_response
 
-            def read(self):
-                return xml
+                # Run test
+                updates = check_updates(True, False, None, None)
+                self.assertEqual(updates, expected)
 
-            def open(self, url):
-                return self
-
-        # TODO: mock
-        bleachbit.Update.build_opener = FakeOpener
-        for xml, expected in update_tests:
-            updates = check_updates(True, False, None, None)
-            self.assertEqual(updates, expected)
-        bleachbit.Update.build_opener = original_open
-
-    def test_UpdateCheck_real_network(self):
-        """Unit tests for class UpdateCheck using real network"""
-
-        # real network
+    def test_check_updates_real_network(self):
+        """Unit test for function check_updates() using real network"""
         for update in check_updates(True, False, None, None):
             if not update:
                 continue
-            ver = update[0]
-            url = update[1]
+            ver, url = update
             self.assertIsInstance(ver, (type(None), str))
             self.assertIsInstance(url, (type(None), str))
 
-    def test_UpdateCheck_real(self):
-        """Unit test for class UpdateCheck with bad network address"""
+    def test_check_updates_bad_url(self):
+        """Unit test for function check_updates() with bad network address"""
         # expect connection failure
         preserve_url = bleachbit.update_check_url
         for url in ('http://localhost/doesnotexist', 'https://httpstat.us/500'):
@@ -93,36 +87,14 @@ class UpdateTestCase(common.BleachbitTestCase):
                 ())
         bleachbit.update_check_url = preserve_url
 
-    def test_get_gtk_version(self):
-        """Unit test for get_gtk_version()"""
-        gtk_ver = get_gtk_version()
-        self.assertIsInstance(gtk_ver, str)
-        self.assertRegex(gtk_ver, r"^\d+\.\d+\.\d+$")
-
-    def test_get_ip_for_url(self):
-        """Unit test for get_ip_for_url()"""
-        for good_url in ('https://www.example.com', bleachbit.update_check_url):
-            ip_str = get_ip_for_url(good_url)
-            import ipaddress
-            ip = ipaddress.ip_address(ip_str)
-        for bad_url in (None, '', 'https://test.invalid'):
-            ret = get_ip_for_url(bad_url)
-            self.assertEqual(ret[0], '(', 'get_ip_for_url({})={}'.format(bad_url,ret))
-
     def test_update_url(self):
-        """Check connection to the update URL"""
-        from bleachbit.Update import build_opener
-        opener = build_opener()
-        opener.addheaders = [('accept','text/*')]
-        import urllib
-        try:
-            handle = opener.open(bleachbit.update_check_url)
-        except urllib.error.HTTPError as e:
-            logger.exception('HTTP error, url: %s\nheaders:\n%s', bleachbit.update_check_url, e.headers)
-            raise e
-        doc = handle.read()
+        """Check that the real update URL returns XML"""
+        from bleachbit.Network import fetch_url
+        response = fetch_url(bleachbit.update_check_url)
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue(response.text.startswith('<?xml version='))
         import xml
-        xml.dom.minidom.parseString(doc)
+        xml.dom.minidom.parseString(response.text)
 
     def test_update_winapp2(self):
         from bleachbit import personal_cleaners_dir
@@ -153,12 +125,6 @@ class UpdateTestCase(common.BleachbitTestCase):
         update_winapp2(url, None, print, on_success)
         update_winapp2(url, None, print, expect_failure)
 
-    def test_user_agent(self):
-        """Unit test for method user_agent()"""
-        agent = user_agent()
-        logger.debug("user agent = '%s'", agent)
-        self.assertIsString(agent)
-
     def test_environment(self):
         """Check the sanity of the environment"""
         import http.client
