@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -59,10 +59,8 @@ def get_winapp2():
             logger.info('deleting stale file %s ', fname)
             os.remove(fname)
     if not os.path.exists(fname):
-        with open(fname, 'w', encoding='utf-8') as fobj:
-            import urllib.request
-            txt = urllib.request.urlopen(url).read()
-            fobj.write(txt.decode('utf-8'))
+        from bleachbit.Network import download_url_to_fn
+        download_url_to_fn(url, fname)
     return fname
 
 
@@ -186,12 +184,8 @@ class WinappTestCase(common.BleachbitTestCase):
 
     def ini2cleaner(self, body, do_next=True):
         """Write a minimal Winapp2.ini"""
-        ini = open(self.ini_fn, 'w')
-        ini.write('[someapp]\n')
-        ini.write('LangSecRef=3021\n')
-        ini.write(body)
-        ini.write('\n')
-        ini.close()
+        with open(self.ini_fn, 'w') as ini:
+            ini.write('[someapp]\nLangSecRef=3021\n' + body + '\n')
         self.assertExists(self.ini_fn)
         if do_next:
             return next(Winapp(self.ini_fn).get_cleaners())
@@ -326,15 +320,19 @@ class WinappTestCase(common.BleachbitTestCase):
 
         # reuse this path to store a winapp2.ini file in
         (ini_h, self.ini_fn) = tempfile.mkstemp(
-            suffix='.ini', prefix='winapp2')
+            suffix='.ini', prefix='winapp2', dir=self.tempdir)
         os.close(ini_h)
 
-        # tests
-        # each tuple
-        # 0 = body of winapp2.ini
-        # 1 = .\deleteme.log should exist
-        # 2 = .\deleteme.bak should exist
-        # 3 = sub\deleteme.log should exist
+        # Each tuple contains:
+        #   [0] = Body of winapp2.ini configuration
+        #   [1] = Whether .\deleteme.log should exist after cleaning (True=exists)
+        #   [2] = Whether .\deleteme.bak should exist after cleaning (True=exists)
+        #   [3] = Whether sub\deleteme.log should exist after cleaning (True=exists)
+        #
+        # Path notation:
+        #   %(d)s     - Literal test directory path without a trailing backslash
+        #   %(d)s\\   - Literal directory with a trailing backslash
+        #   %%bbtestdir%% - Literal environment variable that resolves to same test directory
 
         tests = (
             # delete everything in single directory (no children) without
@@ -354,12 +352,13 @@ class WinappTestCase(common.BleachbitTestCase):
             # exclude log delimited by pipe
             ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=FILE|%(d)s|deleteme.log',
              True, False, True),
-            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=FILE|%(d)s\|deleteme.log',
+            # trailing backslash should work the same as without
+            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=FILE|%(d)s\\|deleteme.log',
              True, False, True),
             # delete / exclude patterns with different complexity
             ('FileKey1=%(d)s|*eteme.*\nExcludeKey1=FILE|%(d)s|deleteme.log',
              True, False, True),
-            ('FileKey1=%(d)s|*eteme.*\nExcludeKey1=FILE|%(d)s\|deleteme.log',
+            ('FileKey1=%(d)s|*eteme.*\nExcludeKey1=FILE|%(d)s\\|deleteme.log',
              True, False, True),
             ('FileKey1=%(d)s|*ete*.*\nExcludeKey1=PATH|%(d)s|*ete*.lo?',
              True, False, True),
@@ -376,7 +375,7 @@ class WinappTestCase(common.BleachbitTestCase):
             # exclude everything in folder
             ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s|*.*',
              True, True, True),
-            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s\|*.*',
+            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s\\|*.*',
              True, True, True),
             ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|%(d)s|*.*',
              True, True, True),
@@ -389,7 +388,7 @@ class WinappTestCase(common.BleachbitTestCase):
             ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%%bbtestdir%%|*.*',
              True, True, True),
             # exclude sub-folder
-            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s\sub',
+            ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s\\sub',
              False, False, True),
             # exclude multiple file types that do not exist
             ('FileKey1=%(d)s|deleteme.*\nExcludeKey1=PATH|%(d)s|*.exe;*.dll',
@@ -400,16 +399,16 @@ class WinappTestCase(common.BleachbitTestCase):
             ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|%(d)s|*.bak;*.log',
              True, True, True),
             # multiple ExcludeKey, neither of which should do anything
-            ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|c:\\doesnotexist|*.*\nExcludeKey2=PATH|c:\\alsodoesnotexist|*.*',
+            ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|c:\\doesnotexist|*.*\nExcludeKey2=PATH|c:\\alsodoesnotexist\\|*.*',
              False, False, False),
             # multiple ExcludeKey, the first should work
-            ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|%(d)s|*.log\nExcludeKey2=PATH|c:\\alsodoesnotexist\|*.*',
+            ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|%(d)s|*.log\nExcludeKey2=PATH|c:\\alsodoesnotexist\\|*.*',
              True, False, True),
             # multiple ExcludeKey, both should work
             ('FileKey1=%(d)s|deleteme.*|RECURSE\nExcludeKey1=PATH|%(d)s|*.log\nExcludeKey2=PATH|%(d)s|*.bak',
              True, True, True),
             # glob should exclude the directory called 'sub'
-            ('FileKey1=%(d)s|*.*\nExcludeKey1=PATH|%(d)s\s*',
+            ('FileKey1=%(d)s|*.*\nExcludeKey1=PATH|%(d)s\\s*',
              False, False, True),
         )
 
@@ -452,8 +451,10 @@ class WinappTestCase(common.BleachbitTestCase):
         # clean their targets.
 
         filename = os.path.join('C:\\', 'deleteme.txt')
-        open(filename, 'w').close()
-        self.assertExists(filename)
+        try:
+            common.touch_file(filename)
+        except PermissionError:
+            self.skipTest('Permission denied: run as administrator')
 
         with mock.patch('os.path.lexists') as mock_lexists:
             cleaner = self.ini2cleaner('FileKey1=%SystemDrive%|deleteme.txt')
@@ -472,23 +473,30 @@ class WinappTestCase(common.BleachbitTestCase):
         # position 2: path of top-folder which should have been deleted
         tests = (
             # Refer to directory directly (i.e., without a glob).
-            (r'FileKey1=%s\dir_c|*.*|REMOVESELF' % self.tempdir, False, r'%s\dir_c' % self.tempdir),
+            (r'FileKey1=%s\dir_c|*.*|REMOVESELF' %
+             self.tempdir, False, r'%s\dir_c' % self.tempdir),
             # Refer to file that exists. This is invalid, so nothing happens.
             (r'FileKey1=%s\dir_c\submarine_sandwich.log|*.*|REMOVESELF' %
              self.tempdir, True, ''),
-            (r'FileKey1=%s\dir_c\submarine*|*.*|REMOVESELF' % self.tempdir, True, ''),
+            (r'FileKey1=%s\dir_c\submarine*|*.*|REMOVESELF' %
+             self.tempdir, True, ''),
             # Refer to path that does not exist, so nothing happens.
             (r'FileKey1=%s\dir_c\doesnotexist.log|*.*|REMOVESELF' %
              self.tempdir, True, ''),
             # Refer by glob to both a file and directory (which both start with `sub`).
             # This should affect only the directory.
-            (r'FileKey1=%s\dir_a\sub*|*.*|REMOVESELF' % self.tempdir, True, r'%s\dir_a\subdir' % self.tempdir),
+            (r'FileKey1=%s\dir_a\sub*|*.*|REMOVESELF' %
+             self.tempdir, True, r'%s\dir_a\subdir' % self.tempdir),
             # glob in middle of directory path with whole directory entry
-            (r'FileKey1=%s\*c\subdir|*.*|REMOVESELF' % self.tempdir, False, r'%s\dir_c\subdir' % self.tempdir),
-            (r'FileKey1=%s\*doesnotexist\subdir|*.*|REMOVESELF' % self.tempdir, True, ''),
+            (r'FileKey1=%s\*c\subdir|*.*|REMOVESELF' %
+             self.tempdir, False, r'%s\dir_c\subdir' % self.tempdir),
+            (r'FileKey1=%s\*doesnotexist\subdir|*.*|REMOVESELF' %
+             self.tempdir, True, ''),
             # glob at end of path
-            (r'FileKey1=%s\dir_c\sub*|*.*|REMOVESELF' % self.tempdir, False, r'%s\dir_c\subdir' % self.tempdir),
-            (r'FileKey1=%s\dir_c\doesnotexist*|*.*|REMOVESELF' % self.tempdir, True, '')
+            (r'FileKey1=%s\dir_c\sub*|*.*|REMOVESELF' %
+             self.tempdir, False, r'%s\dir_c\subdir' % self.tempdir),
+            (r'FileKey1=%s\dir_c\doesnotexist*|*.*|REMOVESELF' %
+             self.tempdir, True, '')
         )
 
         (ini_h, self.ini_fn) = tempfile.mkstemp(
@@ -530,6 +538,26 @@ class WinappTestCase(common.BleachbitTestCase):
         for test in tests:
             self.assertEqual(section2option(test[0]), test[1])
 
+    def test_section_not_found(self):
+        """Test a section that is found"""
+        tmp_ini_fn = os.path.join(self.tempdir, 'section_not_found.ini')
+
+        with open(tmp_ini_fn, 'w') as f:
+            f.write(r"""[Initech TPS Reports *]
+LangSecRef=3023
+DetectFile=%CommonAppData%\Initech\TPSReports
+FileKey1=%CommonAppData%\Initech\TPSReports\Logs|*.*""")
+
+        with mock.patch.object(Winapp, 'detect', return_value=True) as mock_detect:
+            winapps = Winapp(tmp_ini_fn)
+            cleaner_list = list(winapps.get_cleaners())
+            self.assertEqual(len(cleaner_list), 1)
+            self.assertEqual(cleaner_list[0].get_name(), 'Multimedia')
+            this_option = [x for x in cleaner_list[0].get_options()][0]
+            self.assertEqual(this_option[0], 'initech_tps_reports')
+            self.assertEqual(this_option[1], 'Initech TPS Reports')
+            mock_detect.assert_called()
+
     def test_many_patterns(self):
         """Test a cleaner like Steam Installers and related performance improvement
 
@@ -542,9 +570,9 @@ class WinappTestCase(common.BleachbitTestCase):
         print('Making %d files in each of %d directories.' %
               (file_count, dir_count))
         tmp_dir = tempfile.mkdtemp()
-        for i_d in range(1, dir_count+1):
+        for i_d in range(1, dir_count + 1):
             sub_dir = os.path.join(tmp_dir, 'dir%d' % i_d)
-            for i_f in range(1, file_count+1):
+            for i_f in range(1, file_count + 1):
                 tmp_fn = os.path.join(sub_dir, 'file%d' % i_f)
                 common.touch_file(tmp_fn)
 
@@ -563,7 +591,7 @@ class WinappTestCase(common.BleachbitTestCase):
         t0 = time.time()
         self.run_all(cleaner, False)
         t1 = time.time()
-        print('Elapsed time in preview: %.4f seconds ' % (t1-t0))
+        print('Elapsed time in preview: %.4f seconds ' % (t1 - t0))
 
         # delete
         self.run_all(cleaner, False)
