@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -43,7 +43,8 @@ class AccessDeniedActionAction(ActionProvider):
         # access denied, should fail and continue
         def accessdenied():
             import errno
-            raise OSError(errno.EACCES, 'Permission denied: /foo/bar')
+            raise OSError(
+                errno.EACCES, 'Permission denied: c:\\access\\denied', 'c:\\access\\denied')
         yield Command.Function(None, accessdenied, 'Test access denied')
 
         # real file, should succeed
@@ -58,7 +59,10 @@ class DoesNotExistAction(ActionProvider):
 
     def get_commands(self):
         # non-existent file, should fail and continue
-        yield Command.Delete("doesnotexist")
+        if os.name == 'nt':
+            yield Command.Delete(r"c:\does\not\exist")
+        else:
+            yield Command.Delete("/does/not/exist")
 
         # real file, should succeed
         yield Command.Delete(self.pathname)
@@ -144,7 +148,7 @@ class LockedAction(ActionProvider):
         from bleachbit.FileUtilities import getsize
         # Without admin privileges, this delete fails.
         yield Command.Delete(self.pathname)
-        assert(os.path.exists(self.pathname))
+        assert (os.path.exists(self.pathname))
         fsize = getsize(self.pathname)
         if not fsize == 3:  # Contents is "123"
             raise RuntimeError('Locked file has size %dB (not 3B)' % fsize)
@@ -174,7 +178,6 @@ class RuntimeErrorAction(ActionProvider):
 
 
 class TruncateTestAction(ActionProvider):
-
     action_key = 'truncate.test'
 
     def __init__(self, action_element):
@@ -226,11 +229,22 @@ class WorkerTestCase(common.BleachbitTestCase):
 
     def test_AccessDenied(self):
         """Test Worker using Action.AccessDeniedAction"""
-        self.action_test_helper('access.denied', 0, 1, 4096, 1, 3, 1)
+        with self.assertLogs(level='ERROR') as log_context:
+            self.action_test_helper('access.denied', 0, 1, 4096, 1, 3, 1)
+            if os.name == 'nt':
+                self.assertIn('Access denied: c:\\access\\denied',
+                              log_context.output[0])
+                self.assertNotIn('\\\\', log_context.output[0])
 
     def test_DoesNotExist(self):
         """Test Worker using Action.DoesNotExistAction"""
-        self.action_test_helper('does.not.exist', 0, 1, 4096, 1, 3, 1)
+        with self.assertLogs(level='ERROR') as log_context:
+            self.action_test_helper('does.not.exist', 0, 1, 4096, 1, 3, 1)
+            if os.name == 'nt':
+                # Make sure there is a simple messages without double backslashes.
+                self.assertIn(
+                    'File not found: c:\\does\\not\\exist', log_context.output[0])
+                self.assertNotIn('\\\\', log_context.output[0])
 
     def test_FunctionGenerator(self):
         """Test Worker using Action.FunctionGenerator"""
@@ -268,7 +282,8 @@ class WorkerTestCase(common.BleachbitTestCase):
             'locked', 0, errors_expected, None, None, bytes_expected, total_deleted)
 
     def test_RuntimeError(self):
-        """Test Worker using Action.RuntimeErrorAction
+        """Test Worker using Action.RuntimeErrorAction. It is normal to see a traceback.
+
         The Worker module handles these differently than
         access denied exceptions
         """
