@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -26,17 +26,19 @@ import os
 import shutil
 import sys
 import tempfile
+import time
 import unittest
-import mock
+import warnings
+from unittest import mock
+
 if 'win32' == sys.platform:
     import winreg
     import win32gui
 
-
 import bleachbit
 import bleachbit.Options
 from bleachbit.FileUtilities import extended_path
-from bleachbit.General import sudo_mode
+from bleachbit.General import gc_collect, sudo_mode
 
 
 class BleachbitTestCase(unittest.TestCase):
@@ -45,16 +47,24 @@ class BleachbitTestCase(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        """Create a temporary directory for the testcase"""
+        """Do common setup for the test case
+
+        * Create a temporary directory for the testcase.
+        * Treat warnings as errors.
+        This is also set by environment variable in `Makefile` and
+        `appveyor.yml`.
+        * Patch options paths.
+        """
+        warnings.simplefilter("error")
         cls.tempdir = tempfile.mkdtemp(prefix=cls.__name__)
-        print(cls.tempdir)
         if 'BLEACHBIT_TEST_OPTIONS_DIR' not in os.environ:
             cls._patch_options_paths()
 
     @classmethod
     def _patch_options_paths(cls):
-        to_patch = [('bleachbit.options_dir', cls.tempdir), 
-                    ('bleachbit.options_file', os.path.join(cls.tempdir, "bleachbit.ini")),
+        to_patch = [('bleachbit.options_dir', cls.tempdir),
+                    ('bleachbit.options_file', os.path.join(
+                        cls.tempdir, "bleachbit.ini")),
                     ('bleachbit.personal_cleaners_dir', os.path.join(cls.tempdir, "cleaners"))]
         for target, source in to_patch:
             patcher = mock.patch(target, source)
@@ -65,16 +75,31 @@ class BleachbitTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        """remove the temporary directory"""
-        if os.path.exists(cls.tempdir):
-            shutil.rmtree(cls.tempdir)
+        """Do common teardown for the test case
+
+        * Collect garbage.
+        * Remove the temporary directory.
+        * Restore options paths.
+        """
+        gc_collect()
+        # On Windows, a file may be temporarily locked, so retry.
+        for attempt in range(5):
+            try:
+                if os.path.exists(cls.tempdir):
+                    shutil.rmtree(cls.tempdir)
+                break
+            except PermissionError:
+                if attempt < 4:
+                    time.sleep(1)
+                else:
+                    raise
         if 'BLEACHBIT_TEST_OPTIONS_DIR' not in os.environ:
             cls._stop_patch_options_paths()
-    
+
     @classmethod
     def _stop_patch_options_paths(cls):
         for patcher in cls._patchers:
-            patcher.stop()        
+            patcher.stop()
 
     def setUp(cls):
         """Call before each test method"""
@@ -84,6 +109,7 @@ class BleachbitTestCase(unittest.TestCase):
     #
     # type asserts
     #
+
     def assertIsInteger(self, obj, msg=''):
         self.assertIsInstance(obj, int, msg)
 
@@ -93,6 +119,16 @@ class BleachbitTestCase(unittest.TestCase):
     def assertIsBytes(self, obj, msg=''):
         self.assertIsInstance(obj, bytes, msg)
 
+    def assertIsLanguageCode(self, lang_id, msg=''):
+        self.assertIsInstance(lang_id, str)
+        if lang_id in ('C', 'C.UTF-8', 'C.utf8', 'POSIX'):
+            return
+        self.assertTrue(len(lang_id) >= 2)
+        import re
+        pattern = r'^[a-z]{2,3}(_[A-Z][A-Za-z]{1,3})?(@\w+)?(\.[a-zA-Z][a-zA-Z0-9-]+)?$'
+        self.assertTrue(re.match(pattern, lang_id),
+                        f'Invalid language code format: {lang_id}')
+
     @staticmethod
     def check_exists(func, path):
         try:
@@ -135,11 +171,13 @@ class BleachbitTestCase(unittest.TestCase):
     #
     # file creation functions
     #
-    def write_file(self, filename, contents=b'', mode='wb'):
+    def write_file(self, filename, contents=b'', mode='wb', encoding=None):
         """Create a temporary file, optionally writing contents to it"""
+        if not encoding and mode == 'w':
+            encoding = 'utf-8'
         if not os.path.isabs(filename):
             filename = os.path.join(self.tempdir, filename)
-        with open(extended_path(filename), mode) as f:
+        with open(extended_path(filename), mode, encoding=encoding) as f:
             f.write(contents)
         assert (os.path.exists(extended_path(filename)))
         return filename
@@ -176,9 +214,13 @@ def have_root():
 
 
 def put_env(key, val):
-    """Put an environment variable. None removes the key"""
+    """Put an environment variable. None removes the key
+
+    Returns None
+    """
     if not val:
-        del os.environ[key]
+        if key in os.environ:
+            del os.environ[key]
     else:
         os.environ[key] = val
 
@@ -198,6 +240,16 @@ def skipUnlessWindows(f):
     return unittest.skipUnless('win32' == sys.platform, 'not running on Windows')(f)
 
 
+def test_also_with_sudo(test_func):
+    """
+    Decorator to mark test methods that should be run both normally and with sudo.
+
+    See also `tests/test_with_sudo.py`.
+    """
+    test_func._test_also_with_sudo = True
+    return test_func
+
+
 def touch_file(filename):
     """Create an empty file"""
     dname = os.path.dirname(filename)
@@ -206,7 +258,7 @@ def touch_file(filename):
         os.makedirs(dname)
     import pathlib
     pathlib.Path(filename).touch()
-    assert(os.path.exists(filename))
+    assert (os.path.exists(filename))
 
 
 def validate_result(self, result, really_delete=False):
@@ -239,7 +291,7 @@ def validate_result(self, result, really_delete=False):
 
 def get_winregistry_value(key, subkey):
     try:
-        with winreg.OpenKey(key,  subkey) as hkey:
+        with winreg.OpenKey(key, subkey) as hkey:
             return winreg.QueryValue(hkey, None)
     except FileNotFoundError:
         return None
